@@ -1,15 +1,18 @@
 #!/usr/bin/env python3
 
+import logging
 import re
+from dataclasses import dataclass
+from typing import List, NamedTuple, Optional, Set, Tuple
+
 import ghstack
 import ghstack.git
-import ghstack.shell
 import ghstack.github
+import ghstack.github_utils
 import ghstack.logging
-from ghstack.typing import GitHubNumber, GitHubRepositoryId, GhNumber, GitCommitHash, GitTreeHash
-from typing import List, Optional, NamedTuple, Tuple, Set
-import logging
-from dataclasses import dataclass
+import ghstack.shell
+from ghstack.typing import (GhNumber, GitCommitHash, GitHubNumber,
+                            GitHubRepositoryId, GitTreeHash)
 
 # Either "base", "head" or "orig"; which of the ghstack generated
 # branches this diff corresponds to
@@ -21,6 +24,7 @@ DiffMeta = NamedTuple('DiffMeta', [
     ('title', str),
     ('number', GitHubNumber),
     ('body', str),
+    ('username', str),
     ('ghnum', GhNumber),
     # What Git commit hash we should push to what branch
     ('push_branches', Tuple[Tuple[GitCommitHash, BranchKind], ...]),
@@ -60,8 +64,8 @@ RE_GHSTACK_SOURCE_ID = re.compile(r'^ghstack-source-id: (.+)\n?', re.MULTILINE)
 
 
 # repo layout:
-#   - gh/username/23/base -- what we think GitHub's current tip for commit is
-#   - gh/username/23/head -- what we think base commit for commit is
+#   - gh/username/23/head -- what we think GitHub's current tip for commit is
+#   - gh/username/23/base -- what we think base commit for commit is
 #   - gh/username/23/orig -- the "clean" commit history, i.e., what we're
 #                      rebasing, what you'd like to cherry-pick (???)
 #                      (Maybe this isn't necessary, because you can
@@ -93,6 +97,7 @@ STACK_HEADER = "Stack from [ghstack](https://{github_url}/ezyang/ghstack)"
 class DiffWithGitHubMetadata:
     diff: ghstack.diff.Diff
     number: GitHubNumber
+    username: str
     # Really ought not to be optional, but for BC reasons it might be
     remote_source_id: Optional[str]
     title: str
@@ -102,7 +107,8 @@ class DiffWithGitHubMetadata:
     pull_request_resolved: ghstack.diff.PullRequestResolved
 
 
-def main(msg: Optional[str],
+def main(*,
+         msg: Optional[str],
          username: str,
          github: ghstack.github.GitHubEndpoint,
          update_fields: bool = False,
@@ -114,55 +120,26 @@ def main(msg: Optional[str],
          force: bool = False,
          no_skip: bool = False,
          draft: bool = False,
-         github_url: str = "github.com",
-         default_branch: str = "master",
-         remote_name: str = "origin"
+         github_url: str,
+         remote_name: str
          ) -> List[Optional[DiffMeta]]:
 
     if sh is None:
         # Use CWD
         sh = ghstack.shell.Shell()
 
-    if repo_owner is None or repo_name is None:
-        # Grovel in remotes to figure it out
-        remote_url = sh.git("remote", "get-url", remote_name)
-        while True:
-            match = r'^git@{github_url}:([^/]+)/([^.]+)(?:\.git)?$'.format(
-                github_url=github_url
-            )
-            m = re.match(match, remote_url)
-            if m:
-                repo_owner_nonopt = m.group(1)
-                repo_name_nonopt = m.group(2)
-                break
-            search = r'{github_url}/([^/]+)/([^.]+)'.format(
-                github_url=github_url
-            )
-            m = re.search(search, remote_url)
-            if m:
-                repo_owner_nonopt = m.group(1)
-                repo_name_nonopt = m.group(2)
-                break
-            raise RuntimeError(
-                "Couldn't determine repo owner and name from url: {}"
-                .format(remote_url))
-    else:
-        repo_owner_nonopt = repo_owner
-        repo_name_nonopt = repo_name
+    repo = ghstack.github_utils.get_github_repo_info(
+        github=github,
+        sh=sh,
+        repo_owner=repo_owner,
+        repo_name=repo_name,
+        github_url=github_url,
+        remote_name=remote_name,
+    )
+    repo_owner_nonopt = repo["name_with_owner"]["owner"]
+    repo_name_nonopt = repo["name_with_owner"]["name"]
 
-    # TODO: Cache this guy
-    repo = github.graphql(
-        """
-        query ($owner: String!, $name: String!) {
-            repository(name: $name, owner: $owner) {
-                id
-                isFork
-            }
-        }""",
-        owner=repo_owner_nonopt,
-        name=repo_name_nonopt)["data"]["repository"]
-
-    if repo["isFork"]:
+    if repo["is_fork"]:
         raise RuntimeError(
             "Cowardly refusing to upload diffs to a repository that is a "
             "fork.  ghstack expects '{}' of your Git checkout to point "
@@ -173,6 +150,7 @@ def main(msg: Optional[str],
             "error, please register your complaint on GitHub issues (or edit "
             "this line to delete the check above).".format(remote_name))
     repo_id = repo["id"]
+    default_branch = repo["default_branch"]
 
     sh.git("fetch", remote_name)
     base = GitCommitHash(sh.git("merge-base", f"{remote_name}/{default_branch}", "HEAD"))
@@ -450,7 +428,7 @@ the line 'Pull Request resolved' and then run ghexport again.
 ''')
 
         # TODO: Hmm, I'm not sure why this matches
-        m = re.match(r'gh/[^/]+/([0-9]+)/head$', r['headRefName'])
+        m = re.match(r'gh/([^/]+)/([0-9]+)/head$', r['headRefName'])
         if m is None:
             if is_ghexport:
                 raise RuntimeError('''\
@@ -474,7 +452,8 @@ If you think this is in error, run:
 to disassociate the commit with the pull request, and then try again.
 (This will create a new pull request!)
 '''.format(commit.oid))
-        gh_number = GhNumber(m.group(1))
+        username = m.group(1)
+        gh_number = GhNumber(m.group(2))
 
         # NB: Technically, we don't need to pull this information at
         # all, but it's more convenient to unconditionally edit
@@ -489,7 +468,7 @@ to disassociate the commit with the pull request, and then try again.
         remote_summary = ghstack.git.split_header(
             self.sh.git(
                 "rev-list", "--max-count=1", "--header",
-                self.remote_name + "/" + branch_orig(self.username, gh_number)
+                self.remote_name + "/" + branch_orig(username, gh_number)
             )
         )[0]
         m_remote_source_id = RE_GHSTACK_SOURCE_ID.search(remote_summary.commit_msg())
@@ -501,6 +480,7 @@ to disassociate the commit with the pull request, and then try again.
             body=pr_body,
             closed=r['closed'],
             number=number,
+            username=username,
             ghnum=gh_number,
             remote_source_id=remote_source_id,
             pull_request_resolved=commit.pull_request_resolved,
@@ -515,12 +495,14 @@ to disassociate the commit with the pull request, and then try again.
         """
 
         ghnum = commit.ghnum
+        username = commit.username
 
         self.stack_meta.append(DiffMeta(
             title=commit.title,
             number=commit.number,
             body=commit.body,
             ghnum=ghnum,
+            username=username,
             push_branches=(),
             head_branch=None,
             what='Skipped',
@@ -530,10 +512,10 @@ to disassociate the commit with the pull request, and then try again.
 
         self.base_commit = GitCommitHash(self.sh.git(
             "rev-parse",
-            self.remote_name + "/" + branch_head(self.username, ghnum)))
+            self.remote_name + "/" + branch_head(username, ghnum)))
         self.base_orig = GitCommitHash(self.sh.git(
             "rev-parse",
-            self.remote_name + "/" + branch_orig(self.username, ghnum)
+            self.remote_name + "/" + branch_orig(username, ghnum)
         ))
         self.base_tree = GitTreeHash(self.sh.git(
             "rev-parse", self.base_orig + "^{tree}"
@@ -568,6 +550,8 @@ Since we cannot proceed, ghstack will abort now.
         # multiple machines (you bad bad)
         refs = self.sh.git(
             "for-each-ref",
+            # Use OUR username here, since there's none attached to the
+            # diff
             "refs/remotes/{}/gh/{}".format(self.remote_name, self.username),
             "--format=%(refname)").split()
         max_ref_num = max(int(ref.split('/')[-2]) for ref in refs) \
@@ -580,7 +564,7 @@ Since we cannot proceed, ghstack will abort now.
         # Actually, if there's no change in the tree, stop processing
         if tree == self.base_tree:
             self.ignored_diffs.append((commit, None))
-            logging.warn("Skipping {} {}, as the commit has no changes"
+            logging.warning("Skipping {} {}, as the commit has no changes"
                          .format(commit.oid, title))
             self.stack_meta.append(None)
             return
@@ -650,6 +634,7 @@ Since we cannot proceed, ghstack will abort now.
             number=number,
             body=pr_body,
             ghnum=ghnum,
+            username=self.username,
             push_branches=((new_orig, 'orig'), ),
             head_branch=new_pull,
             what='Created',
@@ -667,6 +652,7 @@ Since we cannot proceed, ghstack will abort now.
         """
 
         commit = elab_commit.diff
+        username = elab_commit.username
         ghnum = elab_commit.ghnum
         number = elab_commit.number
 
@@ -706,6 +692,7 @@ Since we cannot proceed, ghstack will abort now.
                     "up-to-date with remote.")
             else:
                 if local_source_id != elab_commit.remote_source_id and not self.force:
+                    logging.debug(f"elab_commit.remote_source_id = {elab_commit.remote_source_id}")
                     raise RuntimeError(
                         "Cowardly refusing to push an update to GitHub, since it "
                         "looks another source has updated GitHub since you last "
@@ -765,7 +752,7 @@ Since we cannot proceed, ghstack will abort now.
         base_args: Tuple[str, ...]
         orig_base_hash = self.sh.git(
             "rev-parse",
-            self.remote_name + "/" + branch_base(self.username, ghnum))
+            self.remote_name + "/" + branch_base(username, ghnum))
 
         # I vacillated between whether or not we should use the PR
         # body or the literal commit message here.  Right now we use
@@ -781,38 +768,19 @@ Since we cannot proceed, ghstack will abort now.
             base_args = ()
 
         else:
-            # Second, check if our local base (self.base_commit)
-            # added some new commits, but is still rooted on the
-            # old base.
-            #
-            # If so, all we need to do is include the local base
-            # as a parent when we do the merge.
-            is_ancestor = self.sh.git(
-                "merge-base",
-                "--is-ancestor",
-                self.remote_name + "/" + branch_base(self.username, ghnum),
-                self.base_commit, exitcode=True)
-
-            if is_ancestor:
-                new_base = self.base_commit
-
-            else:
-                # If we've gotten here, it means that the new
-                # base and the old base are completely
-                # unrelated.  We'll make a fake commit that
-                # "resets" the tree back to something that makes
-                # sense and merge with that.  This doesn't fix
-                # the fact that we still incorrectly report
-                # the old base as an ancestor of our commit, but
-                # it's better than nothing.
-                new_base = GitCommitHash(self.sh.git(
-                    "commit-tree", self.base_tree,
-                    "-p",
-                    self.remote_name + "/" + branch_base(self.username, ghnum),
-                    "-p", self.base_commit,
-                    input='Update base for {} on "{}"\n\n{}\n\n[ghstack-poisoned]'
-                          .format(self.msg, elab_commit.title,
-                                  non_orig_commit_msg)))
+            # Make a fake commit that
+            # "resets" the tree back to something that makes
+            # sense and merge with that.  This doesn't fix
+            # the fact that we still incorrectly report
+            # the old base as an ancestor of our commit, but
+            # it's better than nothing.
+            new_base = GitCommitHash(self.sh.git(
+                "commit-tree", self.base_tree,
+                "-p",
+                self.remote_name + "/" + branch_base(username, ghnum),
+                input='Update base for {} on "{}"\n\n{}\n\n[ghstack-poisoned]'
+                      .format(self.msg, elab_commit.title,
+                              non_orig_commit_msg)))
 
             base_args = ("-p", new_base)
 
@@ -824,13 +792,13 @@ Since we cannot proceed, ghstack will abort now.
         # Nothing to do, just ignore the diff
         if tree == self.base_tree:
             self.ignored_diffs.append((commit, number))
-            logging.warn("Skipping PR #{} {}, as the commit now has no changes"
+            logging.warning("Skipping PR #{} {}, as the commit now has no changes"
                          .format(number, elab_commit.title))
             return
 
         new_pull = GitCommitHash(self.sh.git(
             "commit-tree", tree,
-            "-p", self.remote_name + "/" + branch_head(self.username, ghnum),
+            "-p", self.remote_name + "/" + branch_head(username, ghnum),
             *base_args,
             input='{} on "{}"\n\n{}\n\n[ghstack-poisoned]'.format(self.msg, elab_commit.title, non_orig_commit_msg)))
 
@@ -869,6 +837,7 @@ Since we cannot proceed, ghstack will abort now.
             # it.
             body=elab_commit.body,
             ghnum=ghnum,
+            username=username,
             push_branches=push_branches,
             head_branch=new_pull,
             what=what,
@@ -975,7 +944,7 @@ Since we cannot proceed, ghstack will abort now.
                     q = base_push_branches
                 else:
                     q = push_branches
-                q.append(push_spec(commit, branch(self.username, s.ghnum, b)))
+                q.append(push_spec(commit, branch(s.username, s.ghnum, b)))
         # Careful!  Don't push master.
         # TODO: These pushes need to be atomic (somehow)
         if base_push_branches:
